from .moonblade import MoonBlade, EventCode
from .router import Router, Node

__title__ = "moonblade"
__description__ = "A Python 3 asynchronous library committed to communicating with LOL server through the LCU API in a simple and flexible way ."
__version__ = "1.0.4"
__author__ = "gfk-sveyigey"
__author_email__ = "gfk_sveyigey@163.com"
__url__ = "https://github.com/gfk-sveyigey/moonblade"


__all__ = [
    "__title__",
    "__description__",
    "__version__",
    "MoonBlade",
    "EventCode",
    "Router",
    "Node",
]